import { BodyCellComponentTemplate } from './BodyCell.stories.js'
import { BaseHeaderCellComponentTemplate } from './HeaderCell.stories.js'
import { IconOnly } from '../Button.stories.js'
import { WithIconLeft } from '../Tag.stories.js'
import { faker } from '@faker-js/faker';

/**
 * A single row component to be used in a table body.
 * 
 * Should be built using `HeaderCell` or `BodyCell` templates/components.
 * 
 * Please ignore the last preview (RowComponentTemplate) - it is only exported to be used in the table component, and cannot display properly on its own.
 * */
export default {
  title: 'Components/Table/Row',
  tags: ['autodocs'],
  argTypes: {
    rowType: {
      control: 'select',
      options: ['body', 'header'],
    },
    numColumns: {
      control: 'number',
      description: 'Number of columns in the table row.',
    },
    sortHeaders: {
      control: 'boolean',
      description: 'Set to true if the table row is a header row to show the sorting arrows.',
    },
    includeButton: {
      control: 'boolean',
      description: 'Set to true to include a button in the table',
    },
    includeTag: {
      control: 'boolean',
      description: 'Set to true to include a tag in the table',
    },
    fullWidth: {
      control: 'boolean',
    },
  },
  args: {
    rowType: 'body',
    numColumns: 4,
    sortHeaders: false,
    includeButton: false,
    includeTag: false,
    fullWidth: false,
  },
}

function generateHeaderCellContent(data, sortHeaders) {
  return data.map((content) => ({ content, sort: sortHeaders, sortDirection: 'down' }))
}

function generateTableCellContent(data, includeButton, includeTag, tagColumn = null) {
  const cells = data.map((content) => ({ content }))
  if (includeTag) {
    // Choose random index to insert tag (except last column) unless a specific column is specified
    const tagIndex = tagColumn ? (tagColumn % data.length) : Math.floor(Math.random() * (cells.length - 1))
    cells.splice(tagIndex, 1, { content: WithIconLeft({ text: faker.lorem.word(), icon: 'check', iconPosition: 'left'}) })
  }
  if (includeButton) {
    cells[cells.length -1] = { content: IconOnly({ icon: 'arrow_forward', iconOnly: true, type: 'rpf-button--tertiary'}) }
  }
  return cells
}

// Standalone row component template
const RowComponentTemplate = (args) => {
  const headerCellContent = Array.from({ length: args.numColumns }, () => faker.lorem.word())
  const cellContent = Array.from({ length: args.numColumns }, () => faker.lorem.words(3))
  const cells = args.rowType === 'header' ? generateHeaderCellContent(headerCellContent, args.sortHeaders) : generateTableCellContent(cellContent, args.includeButton, args.includeTag, args.tagColumn, args.maxCharacterLength)

  return `
    <tr class='rpf-table-row'>
      ${cells.map(cell => args.rowType === 'body' ? BodyCellComponentTemplate(cell) : BaseHeaderCellComponentTemplate(cell)).join('')}
    </tr>`
};

// Wrapper template for the preview with the <table> tag
const PreviewTemplate = (args) => {
  return `
    <table class='rpf-table__table ${args.fullWidth ? 'rpf-table__table--full-width' : ''}'>
      ${RowComponentTemplate(args)}
    </table>
  `;
};

export const Row = PreviewTemplate.bind({})
Row.args = {
  rowType: 'body',
  numColumns: 4,
}

export const BodyCellRow = PreviewTemplate.bind({})
BodyCellRow.args = {
  rowType: 'body',
  numColumns: 5,
}

export const HeaderCellRow = PreviewTemplate.bind({})
HeaderCellRow.args = {
  rowType: 'header',
  numColumns: 5,
}

// Export RowComponentTemplate for table component to use
export { RowComponentTemplate };
